@@ -2,11 +2,12 @@
 
 """API functions for PyStow."""
 
+import sqlite3
 from contextlib import contextmanager
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Mapping, Optional, Sequence, Union
+from typing import TYPE_CHECKING, Any, Generator, Mapping, Optional, Sequence, Union
 
-from .constants import JSON, Opener, Provider
+from .constants import JSON, BytesOpener, Opener, Provider
 from .impl import Module
 
 if TYPE_CHECKING:
@@ -115,7 +116,7 @@ def open(
     name: str,
     mode: str = "r",
     open_kwargs: Optional[Mapping[str, Any]] = None,
-):
+) -> Opener:
     """Open a file that exists already.
 
     :param key:
@@ -143,7 +144,7 @@ def open_gz(
     name: str,
     mode: str = "rt",
     open_kwargs: Optional[Mapping[str, Any]] = None,
-):
+) -> Opener:
     """Open a gzipped file that exists already.
 
     :param key:
@@ -205,7 +206,7 @@ def ensure_custom(
     name: str,
     force: bool = False,
     provider: Provider,
-    **kwargs,
+    **kwargs: Any,
 ) -> Path:
     """Ensure a file is present, and run a custom create function otherwise.
 
@@ -382,7 +383,7 @@ def ensure_open_zip(
     download_kwargs: Optional[Mapping[str, Any]] = None,
     mode: str = "r",
     open_kwargs: Optional[Mapping[str, Any]] = None,
-) -> Opener:
+) -> BytesOpener:
     """Ensure a file is downloaded then open it with :mod:`zipfile`.
 
     :param key:
@@ -480,7 +481,7 @@ def ensure_open_tarfile(
     download_kwargs: Optional[Mapping[str, Any]] = None,
     mode: str = "r",
     open_kwargs: Optional[Mapping[str, Any]] = None,
-) -> Opener:
+) -> BytesOpener:
     """Ensure a tar file is downloaded and open a file inside it.
 
     :param key:
@@ -708,7 +709,7 @@ def dump_df(
     name: str,
     obj: "pd.DataFrame",
     sep: str = "\t",
-    index=False,
+    index: bool = False,
     to_csv_kwargs: Optional[Mapping[str, Any]] = None,
 ) -> None:
     """Dump a dataframe to a TSV file with :mod:`pandas`.
@@ -1197,7 +1198,7 @@ def ensure_tar_df(
     force: bool = False,
     download_kwargs: Optional[Mapping[str, Any]] = None,
     read_csv_kwargs: Optional[Mapping[str, Any]] = None,
-):
+) -> "pd.DataFrame":
     """Download a tar file and open an inner file as a dataframe with :mod:`pandas`.
 
     :param key: The module name
@@ -1241,7 +1242,7 @@ def ensure_tar_xml(
     force: bool = False,
     download_kwargs: Optional[Mapping[str, Any]] = None,
     parse_kwargs: Optional[Mapping[str, Any]] = None,
-):
+) -> "lxml.etree.ElementTree":
     """Download a tar file and open an inner file as an XML with :mod:`lxml`.
 
     :param key: The module name
@@ -1453,7 +1454,7 @@ def dump_rdf(
     obj: "rdflib.Graph",
     format: str = "turtle",
     serialize_kwargs: Optional[Mapping[str, Any]] = None,
-):
+) -> None:
     """Dump an RDF graph to a file with :mod:`rdflib`.
 
     :param key:
@@ -1480,7 +1481,7 @@ def ensure_from_s3(
     s3_key: Union[str, Sequence[str]],
     name: Optional[str] = None,
     force: bool = False,
-    **kwargs,
+    **kwargs: Any,
 ) -> Path:
     """Ensure a file is downloaded.
 
@@ -1512,7 +1513,7 @@ def ensure_from_s3(
     """
     _module = Module.from_key(key, ensure_exists=True)
     return _module.ensure_from_s3(
-        *subkeys, s3_bucket=s3_bucket, s3_key=s3_key, name=name, force=force
+        *subkeys, s3_bucket=s3_bucket, s3_key=s3_key, name=name, force=force, **kwargs
     )
 
 
@@ -1578,7 +1579,7 @@ def ensure_open_sqlite(
     name: Optional[str] = None,
     force: bool = False,
     download_kwargs: Optional[Mapping[str, Any]] = None,
-):
+) -> Generator[sqlite3.Connection, None, None]:
     """Ensure and connect to a SQLite database.
 
     :param key:
@@ -1622,7 +1623,7 @@ def ensure_open_sqlite_gz(
     name: Optional[str] = None,
     force: bool = False,
     download_kwargs: Optional[Mapping[str, Any]] = None,
-):
+) -> Generator[sqlite3.Connection, None, None]:
     """Ensure and connect to a gzipped SQLite database.
 
     :param key:
